@@ -8,8 +8,7 @@ from twisted.python import log
 from twisted.python.failure import Failure
 from twisted.web.util import formatFailure
 
-from buildbot import util
-from buildbot import interfaces
+from buildbot import interfaces, locks
 from buildbot.status import progress
 from buildbot.status.builder import SUCCESS, WARNINGS, FAILURE, SKIPPED, \
      EXCEPTION
@@ -434,7 +433,7 @@ class RemoteShellCommand(LoggedRemoteCommand):
 
     def __init__(self, workdir, command, env=None, 
                  want_stdout=1, want_stderr=1,
-                 timeout=20*60, logfiles={}, **kwargs):
+                 timeout=20*60, logfiles={}, usePTY="slave-config"):
         """
         @type  workdir: string
         @param workdir: directory where the command ought to run,
@@ -487,6 +486,7 @@ class RemoteShellCommand(LoggedRemoteCommand):
                 'want_stderr': want_stderr,
                 'logfiles': logfiles,
                 'timeout': timeout,
+                'usePTY': usePTY,
                 }
         LoggedRemoteCommand.__init__(self, "shell", args)
 
@@ -503,7 +503,7 @@ class RemoteShellCommand(LoggedRemoteCommand):
         return LoggedRemoteCommand.start(self)
 
     def __repr__(self):
-        return "<RemoteShellCommand '%s'>" % self.command
+        return "<RemoteShellCommand '%s'>" % repr(self.command)
 
 class BuildStep:
     """
@@ -545,11 +545,14 @@ class BuildStep:
     # immediately, the others will be taken into consideration when
     # determining the overall build status.
     #
+    # steps that are makred as alwaysRun will be run regardless of the outcome
+    # of previous steps (especially steps with haltOnFailure=True)
     haltOnFailure = False
     flunkOnWarnings = False
     flunkOnFailure = False
     warnOnWarnings = False
     warnOnFailure = False
+    alwaysRun = False
 
     # 'parms' holds a list of all the parameters we care about, to allow
     # users to instantiate a subclass of BuildStep with a mixture of
@@ -564,7 +567,9 @@ class BuildStep:
              'flunkOnFailure',
              'warnOnWarnings',
              'warnOnFailure',
+             'alwaysRun',
              'progressMetrics',
+             'doStepIf',
              ]
 
     name = "generic"
@@ -574,6 +579,8 @@ class BuildStep:
     build = None
     step_status = None
     progress = None
+    # doStepIf can be False, True, or a function that returns False or True
+    doStepIf = True
 
     def __init__(self, **kwargs):
         self.factory = (self.__class__, dict(kwargs))
@@ -599,11 +606,11 @@ class BuildStep:
         self.buildslave = buildslave
 
     def setDefaultWorkdir(self, workdir):
-        # the Build calls this just after __init__ and setDefaultWorkdir.
-        # ShellCommand and variants use a slave-side workdir, but some other
-        # steps do not. Subclasses which use a workdir should use the value
-        # set by this method unless they were constructed with something more
-        # specific.
+        # The Build calls this just after __init__().  ShellCommand
+        # and variants use a slave-side workdir, but some other steps
+        # do not. Subclasses which use a workdir should use the value
+        # set by this method unless they were constructed with
+        # something more specific.
         pass
 
     def addFactoryArguments(self, **kwargs):
@@ -632,8 +639,8 @@ class BuildStep:
     def getProperty(self, propname):
         return self.build.getProperty(propname)
 
-    def setProperty(self, propname, value):
-        self.build.setProperty(propname, value)
+    def setProperty(self, propname, value, source="Step"):
+        self.build.setProperty(propname, value, source)
 
     def startStep(self, remote):
         """Begin the step. This returns a Deferred that will fire when the
@@ -663,12 +670,18 @@ class BuildStep:
         self.remote = remote
         self.deferred = defer.Deferred()
         # convert all locks into their real form
-        self.locks = [self.build.builder.botmaster.getLockByID(l)
-                      for l in self.locks]
+        lock_list = []
+        for access in self.locks:
+            if not isinstance(access, locks.LockAccess):
+                # Buildbot 0.7.7 compability: user did not specify access
+                access = access.defaultAccess()
+            lock = self.build.builder.botmaster.getLockByID(access.lockid)
+            lock_list.append((lock, access))
+        self.locks = lock_list
         # then narrow SlaveLocks down to the slave that this build is being
         # run on
-        self.locks = [l.getLock(self.build.slavebuilder) for l in self.locks]
-        for l in self.locks:
+        self.locks = [(l.getLock(self.build.slavebuilder), la) for l, la in self.locks]
+        for l, la in self.locks:
             if l in self.build.locks:
                 log.msg("Hey, lock %s is claimed by both a Step (%s) and the"
                         " parent Build (%s)" % (l, self, self.build))
@@ -681,15 +694,15 @@ class BuildStep:
         log.msg("acquireLocks(step %s, locks %s)" % (self, self.locks))
         if not self.locks:
             return defer.succeed(None)
-        for lock in self.locks:
-            if not lock.isAvailable():
+        for lock, access in self.locks:
+            if not lock.isAvailable(access):
                 log.msg("step %s waiting for lock %s" % (self, lock))
-                d = lock.waitUntilMaybeAvailable(self)
+                d = lock.waitUntilMaybeAvailable(self, access)
                 d.addCallback(self.acquireLocks)
                 return d
         # all locks are available, claim them all
-        for lock in self.locks:
-            lock.claim(self)
+        for lock, access in self.locks:
+            lock.claim(self, access)
         return defer.succeed(None)
 
     def _startStep_2(self, res):
@@ -697,10 +710,20 @@ class BuildStep:
             self.progress.start()
         self.step_status.stepStarted()
         try:
-            skip = self.start()
+            skip = None
+            if isinstance(self.doStepIf, bool):
+                if not self.doStepIf:
+                    skip = SKIPPED
+            elif not self.doStepIf(self):
+                skip = SKIPPED
+
+            if skip is None:
+                skip = self.start()
+
             if skip == SKIPPED:
-                reactor.callLater(0, self.releaseLocks)
-                reactor.callLater(0, self.deferred.callback, SKIPPED)
+                # this return value from self.start is a shortcut
+                # to finishing the step immediately
+                reactor.callLater(0, self.finished, SKIPPED)
         except:
             log.msg("BuildStep.startStep exception in .start")
             self.failed(Failure())
@@ -719,7 +742,6 @@ class BuildStep:
         As the step runs, it should send status information to the
         BuildStepStatus::
 
-          self.step_status.setColor('red')
           self.step_status.setText(['compile', 'failed'])
           self.step_status.setText2(['4', 'warnings'])
 
@@ -755,7 +777,15 @@ class BuildStep:
         If the step decides it does not need to be run, start() can return
         the constant SKIPPED. This fires the callback immediately: it is not
         necessary to call .finished yourself. This can also indicate to the
-        status-reporting mechanism that this step should not be displayed."""
+        status-reporting mechanism that this step should not be displayed.
+
+        A step can be configured to only run under certain conditions.  To
+        do this, set the step's doStepIf to a boolean value, or to a function
+        that returns a boolean value.  If the value or function result is
+        False, then the step will return SKIPPED without doing anything,
+        otherwise the step will be executed normally.  If you set doStepIf
+        to a function, that function should accept one parameter, which will
+        be the Step object itself."""
         
         raise NotImplementedError("your subclass must implement this method")
 
@@ -770,8 +800,8 @@ class BuildStep:
 
     def releaseLocks(self):
         log.msg("releaseLocks(%s): %s" % (self, self.locks))
-        for lock in self.locks:
-            lock.release(self)
+        for lock, access in self.locks:
+            lock.release(self, access)
 
     def finished(self, results):
         if self.progress:
@@ -792,7 +822,6 @@ class BuildStep:
             self.addHTMLLog("err.html", formatFailure(why))
             self.addCompleteLog("err.text", why.getTraceback())
             # could use why.getDetailedTraceback() for more information
-            self.step_status.setColor("purple")
             self.step_status.setText([self.name, "exception"])
             self.step_status.setText2([self.name])
             self.step_status.stepFinished(EXCEPTION)
@@ -944,9 +973,9 @@ class LoggingBuildStep(BuildStep):
         @param cmd: a suitable RemoteCommand which will be launched, with
                     all output being put into our self.stdio_log LogFile
         """
-        log.msg("ShellCommand.startCommand(cmd=%s)", (cmd,))
+        log.msg("ShellCommand.startCommand(cmd=%s)" % (cmd,))
+        log.msg("  cmd.args = %r" % (cmd.args))
         self.cmd = cmd # so we can interrupt it
-        self.step_status.setColor("yellow")
         self.step_status.setText(self.describe(False))
 
         # stdio is the first log
@@ -991,7 +1020,6 @@ class LoggingBuildStep(BuildStep):
 
     def checkDisconnect(self, f):
         f.trap(error.ConnectionLost)
-        self.step_status.setColor("red")
         self.step_status.setText(self.describe(True) +
                                  ["failed", "slave", "lost"])
         self.step_status.setText2(["failed", "slave", "lost"])
@@ -1076,62 +1104,14 @@ class LoggingBuildStep(BuildStep):
                 return self.getText2(cmd, results)
         return []
 
-    def getColor(self, cmd, results):
-        assert results in (SUCCESS, WARNINGS, FAILURE)
-        if results == SUCCESS:
-            return "green"
-        elif results == WARNINGS:
-            return "orange"
-        else:
-            return "red"
-
     def setStatus(self, cmd, results):
         # this is good enough for most steps, but it can be overridden to
         # get more control over the displayed text
-        self.step_status.setColor(self.getColor(cmd, results))
         self.step_status.setText(self.getText(cmd, results))
         self.step_status.setText2(self.maybeGetText2(cmd, results))
 
-class _BuildPropertyMapping:
-    def __init__(self, build):
-        self.build = build
-    def __getitem__(self, name):
-        p = self.build.getProperty(name)
-        if p is None:
-            p = ""
-        return p
+# (WithProeprties used to be available in this module)
+from buildbot.process.properties import WithProperties
+_hush_pyflakes = [WithProperties]
+del _hush_pyflakes
 
-class WithProperties(util.ComparableMixin):
-    """This is a marker class, used in ShellCommand's command= argument to
-    indicate that we want to interpolate a build property.
-    """
-
-    compare_attrs = ('fmtstring', 'args')
-
-    def __init__(self, fmtstring, *args):
-        self.fmtstring = fmtstring
-        self.args = args
-
-    def render(self, build):
-        pmap = _BuildPropertyMapping(build)
-        if self.args:
-            strings = []
-            for name in self.args:
-                strings.append(pmap[name])
-            s = self.fmtstring % tuple(strings)
-        else:
-            s = self.fmtstring % pmap
-        return s
-
-def render_properties(s, build):
-    """Return a string based on s and build that is suitable for use
-    in a running BuildStep.  If s is a string, return s.  If s is a
-    WithProperties object, return the result of s.render(build).
-    Otherwise, return str(s).
-    """
-    if isinstance(s, (str, unicode)):
-        return s
-    elif isinstance(s, WithProperties):
-        return s.render(build)
-    else:
-        return str(s)
