@@ -13,12 +13,17 @@ from buildbot.interfaces import IControl, IStatusReceiver
 from buildbot.status.web.base import HtmlResource, Box, \
      build_get_class, ICurrentBox, OneLineMixin, map_branches, \
      make_stop_form, make_force_build_form
+from buildbot.status.web.feeds import Rss20StatusResource, \
+     Atom10StatusResource
 from buildbot.status.web.waterfall import WaterfallStatusResource
+from buildbot.status.web.grid import GridStatusResource, TransposedGridStatusResource
 from buildbot.status.web.changes import ChangesResource
 from buildbot.status.web.builder import BuildersResource
+from buildbot.status.web.buildstatus import BuildStatusStatusResource 
 from buildbot.status.web.slaves import BuildSlavesResource
 from buildbot.status.web.xmlrpc import XMLRPCServer
 from buildbot.status.web.about import AboutBuildbot
+from buildbot.status.web.auth import IAuth, AuthFailResource
 
 # this class contains the status services (WebStatus and the older Waterfall)
 # which can be put in c['status']. It also contains some of the resources
@@ -67,7 +72,7 @@ def getLastNBuilds(status, numbuilds, builders=[], branches=[]):
 
 
 # /one_line_per_build
-#  accepts builder=, branch=, numbuilds=
+#  accepts builder=, branch=, numbuilds=, reload=
 class OneLinePerBuild(HtmlResource, OneLineMixin):
     """This shows one line per build, combining all builders together. Useful
     query arguments:
@@ -75,6 +80,7 @@ class OneLinePerBuild(HtmlResource, OneLineMixin):
     numbuilds=: how many lines to display
     builder=: show only builds for this builder. Multiple builder= arguments
               can be used to see builds from any builder in the set.
+    reload=: reload the page after this many seconds
     """
 
     title = "Recent Builds"
@@ -86,24 +92,43 @@ class OneLinePerBuild(HtmlResource, OneLineMixin):
     def getChild(self, path, req):
         status = self.getStatus(req)
         builder = status.getBuilder(path)
-        return OneLinePerBuildOneBuilder(builder)
+        return OneLinePerBuildOneBuilder(builder, numbuilds=self.numbuilds)
+
+    def get_reload_time(self, request):
+        if "reload" in request.args:
+            try:
+                reload_time = int(request.args["reload"][0])
+                return max(reload_time, 15)
+            except ValueError:
+                pass
+        return None
+
+    def head(self, request):
+        head = ''
+        reload_time = self.get_reload_time(request)
+        if reload_time is not None:
+            head += '<meta http-equiv="refresh" content="%d">\n' % reload_time
+        return head
 
     def body(self, req):
         status = self.getStatus(req)
+        control = self.getControl(req)
         numbuilds = int(req.args.get("numbuilds", [self.numbuilds])[0])
         builders = req.args.get("builder", [])
         branches = [b for b in req.args.get("branch", []) if b]
 
         g = status.generateFinishedBuilds(builders, map_branches(branches),
-                                          numbuilds)
+                                          numbuilds, max_search=numbuilds)
 
         data = ""
 
         # really this is "up to %d builds"
+        html_branches = map(html.escape, branches)
         data += "<h1>Last %d finished builds: %s</h1>\n" % \
-                (numbuilds, ", ".join(branches))
+                (numbuilds, ", ".join(html_branches))
         if builders:
-            data += ("<p>of builders: %s</p>\n" % (", ".join(builders)))
+            html_builders = map(html.escape, builders)
+            data += ("<p>of builders: %s</p>\n" % (", ".join(html_builders)))
         data += "<ul>\n"
         got = 0
         building = False
@@ -121,12 +146,15 @@ class OneLinePerBuild(HtmlResource, OneLineMixin):
             data += " <li>No matching builds found</li>\n"
         data += "</ul>\n"
 
-        if building:
-            stopURL = "builders/_all/stop"
-            data += make_stop_form(stopURL, True, "Builds")
-        if online:
-            forceURL = "builders/_all/force"
-            data += make_force_build_form(forceURL, True)
+        if control is not None:
+            if building:
+                stopURL = "builders/_all/stop"
+                data += make_stop_form(stopURL, self.isUsingUserPasswd(req),
+                                       True, "Builds")
+            if online:
+                forceURL = "builders/_all/force"
+                data += make_force_build_form(forceURL,
+                                              self.isUsingUserPasswd(req), True)
 
         return data
 
@@ -153,8 +181,9 @@ class OneLinePerBuildOneBuilder(HtmlResource, OneLineMixin):
                                                 numbuilds)
 
         data = ""
+        html_branches = map(html.escape, branches)
         data += ("<h1>Last %d builds of builder %s: %s</h1>\n" %
-                 (numbuilds, self.builder_name, ", ".join(branches)))
+                 (numbuilds, self.builder_name, ", ".join(html_branches)))
         data += "<ul>\n"
         got = 0
         for build in g:
@@ -169,7 +198,7 @@ class OneLinePerBuildOneBuilder(HtmlResource, OneLineMixin):
 # /one_box_per_builder
 #  accepts builder=, branch=
 class OneBoxPerBuilder(HtmlResource):
-    """This shows a narrow table with one row per build. The leftmost column
+    """This shows a narrow table with one row per builder. The leftmost column
     contains the builder name. The next column contains the results of the
     most recent build. The right-hand column shows the builder's current
     activity.
@@ -182,13 +211,15 @@ class OneBoxPerBuilder(HtmlResource):
 
     def body(self, req):
         status = self.getStatus(req)
+        control = self.getControl(req)
 
         builders = req.args.get("builder", status.getBuilderNames())
         branches = [b for b in req.args.get("branch", []) if b]
 
         data = ""
 
-        data += "<h2>Latest builds: %s</h2>\n" % ", ".join(branches)
+        html_branches = map(html.escape, branches)
+        data += "<h2>Latest builds: %s</h2>\n" % ", ".join(html_branches)
         data += "<table>\n"
 
         building = False
@@ -213,7 +244,7 @@ class OneBoxPerBuilder(HtmlResource):
                     label = "#%d" % b.getNumber()
                 text = ['<a href="%s">%s</a>' % (url, label)]
                 text.extend(b.getText())
-                box = Box(text, b.getColor(),
+                box = Box(text,
                           class_="LastBuild box %s" % build_get_class(b))
                 data += box.td(align="center")
             else:
@@ -230,12 +261,15 @@ class OneBoxPerBuilder(HtmlResource):
 
         data += "</table>\n"
 
-        if building:
-            stopURL = "builders/_all/stop"
-            data += make_stop_form(stopURL, True, "Builds")
-        if online:
-            forceURL = "builders/_all/force"
-            data += make_force_build_form(forceURL, True)
+        if control is not None:
+            if building:
+                stopURL = "builders/_all/stop"
+                data += make_stop_form(stopURL, self.isUsingUserPasswd(req),
+                                       True, "Builds")
+            if online:
+                forceURL = "builders/_all/force"
+                data += make_force_build_form(forceURL,
+                                              self.isUsingUserPasswd(req), True)
 
         return data
 
@@ -280,6 +314,17 @@ class WebStatus(service.MultiService):
                   to individual changes, builders, builds, steps, and logs.
                   A number of query-arguments can be added to influence
                   the display.
+     /rss : a rss feed summarizing all failed builds. The same
+            query-arguments used by 'waterfall' can be added to
+            influence the feed output.
+     /atom : an atom feed summarizing all failed builds. The same
+             query-arguments used by 'waterfall' can be added to
+             influence the feed output.
+     /grid : another summary display that shows a grid of builds, with
+             sourcestamps on the x axis, and builders on the y.  Query
+             arguments similar to those for the waterfall can be added.
+     /tgrid : similar to the grid display, but the commits are down the
+              left side, and the build hosts are across the top.
      /builders/BUILDERNAME: a page summarizing the builder. This includes
                             references to the Schedulers that feed it,
                             any builds currently in the queue, which
@@ -307,12 +352,13 @@ class WebStatus(service.MultiService):
      /xmlrpc : (not yet implemented) an XMLRPC server with build status
 
 
-    All URLs for pages which are not defined here are used to look for files
-    in BASEDIR/public_html/ , which means that /robots.txt or /buildbot.css
-    or /favicon.ico can be placed in that directory.
+    All URLs for pages which are not defined here are used to look
+    for files in PUBLIC_HTML, which defaults to BASEDIR/public_html.
+    This means that /robots.txt or /buildbot.css or /favicon.ico can
+    be placed in that directory.
 
     If an index file (index.html, index.htm, or index, in that order) is
-    present in public_html/, it will be used for the root resource. If not,
+    present in PUBLIC_HTML, it will be used for the root resource. If not,
     the default behavior is to put a redirection to the /waterfall page.
 
     All of the resources provided by this service use relative URLs to reach
@@ -321,7 +367,7 @@ class WebStatus(service.MultiService):
     bottom.
 
     This webserver defines class attributes on elements so they can be styled
-    with CSS stylesheets. All pages pull in public_html/buildbot.css, and you
+    with CSS stylesheets. All pages pull in PUBLIC_HTML/buildbot.css, and you
     can cause additional stylesheets to be loaded by adding a suitable <link>
     to the WebStatus instance's .head_elements attribute.
 
@@ -350,7 +396,8 @@ class WebStatus(service.MultiService):
     # not (we'd have to do a recursive traversal of all children to discover
     # all the changes).
 
-    def __init__(self, http_port=None, distrib_port=None, allowForce=False):
+    def __init__(self, http_port=None, distrib_port=None, allowForce=False,
+                 public_html="public_html", site=None, numbuilds=20, auth=None):
         """Run a web server that provides Buildbot status.
 
         @type  http_port: int or L{twisted.application.strports} string
@@ -383,8 +430,29 @@ class WebStatus(service.MultiService):
                              'unix:/home/buildbot/.twistd-web-pb'. Providing
                              a non-absolute pathname will probably confuse
                              the strports parser.
+
         @param allowForce: boolean, if True then the webserver will allow
                            visitors to trigger and cancel builds
+
+        @param public_html: the path to the public_html directory for this display,
+                            either absolute or relative to the basedir.  The default
+                            is 'public_html', which selects BASEDIR/public_html.
+
+        @type site: None or L{twisted.web.server.Site}
+        @param site: Use this if you want to define your own object instead of
+                     using the default.`
+
+        @type numbuilds: int
+        @param numbuilds: Default number of entries in lists at the /one_line_per_build
+        and /builders/FOO URLs.  This default can be overriden both programatically ---
+        by passing the equally named argument to constructors of OneLinePerBuildOneBuilder
+        and OneLinePerBuild --- and via the UI, by tacking ?numbuilds=xy onto the URL.
+
+        @type auth: a L{status.web.auth.IAuth} or C{None}
+        @param auth: an object that performs authentication to restrict access
+                     to the C{allowForce} features. Ignored if C{allowForce}
+                     is not C{True}. If C{auth} is C{None}, people can force or
+                     stop builds without auth.
         """
 
         service.MultiService.__init__(self)
@@ -398,14 +466,28 @@ class WebStatus(service.MultiService):
                 distrib_port = "unix:%s" % distrib_port
         self.distrib_port = distrib_port
         self.allowForce = allowForce
+        self.public_html = public_html
 
-        # this will be replaced once we've been attached to a parent (and
-        # thus have a basedir and can reference BASEDIR/public_html/)
-        root = static.Data("placeholder", "text/plain")
-        self.site = server.Site(root)
+        if self.allowForce and auth:
+            assert IAuth.providedBy(auth)
+            self.auth = auth
+        else:
+            if auth:
+                log.msg("Warning: Ignoring authentication. allowForce must be"
+                        " set to True use this")
+            self.auth = None
+
+        # If we were given a site object, go ahead and use it.
+        if site:
+            self.site = site
+        else:
+            # this will be replaced once we've been attached to a parent (and
+            # thus have a basedir and can reference BASEDIR)
+            root = static.Data("placeholder", "text/plain")
+            self.site = server.Site(root)
         self.childrenToBeAdded = {}
 
-        self.setupUsualPages()
+        self.setupUsualPages(numbuilds=numbuilds)
 
         # the following items are accessed by HtmlResource when it renders
         # each page.
@@ -428,17 +510,22 @@ class WebStatus(service.MultiService):
             s = strports.service(self.distrib_port, f)
             s.setServiceParent(self)
 
-    def setupUsualPages(self):
+    def setupUsualPages(self, numbuilds):
         #self.putChild("", IndexOrWaterfallRedirection())
         self.putChild("waterfall", WaterfallStatusResource())
+        self.putChild("grid", GridStatusResource())
+        self.putChild("tgrid", TransposedGridStatusResource())
         self.putChild("builders", BuildersResource()) # has builds/steps/logs
         self.putChild("changes", ChangesResource())
         self.putChild("buildslaves", BuildSlavesResource())
+        self.putChild("buildstatus", BuildStatusStatusResource())
         #self.putChild("schedulers", SchedulersResource())
-        self.putChild("one_line_per_build", OneLinePerBuild())
+        self.putChild("one_line_per_build",
+                      OneLinePerBuild(numbuilds=numbuilds))
         self.putChild("one_box_per_builder", OneBoxPerBuilder())
         self.putChild("xmlrpc", XMLRPCServer())
         self.putChild("about", AboutBuildbot())
+        self.putChild("authfail", AuthFailResource())
 
     def __repr__(self):
         if self.http_port is None:
@@ -452,12 +539,19 @@ class WebStatus(service.MultiService):
 
     def setServiceParent(self, parent):
         service.MultiService.setServiceParent(self, parent)
+
+        # this class keeps a *separate* link to the buildmaster, rather than
+        # just using self.parent, so that when we are "disowned" (and thus
+        # parent=None), any remaining HTTP clients of this WebStatus will still
+        # be able to get reasonable results.
+        self.master = parent
+
         self.setupSite()
 
     def setupSite(self):
         # this is responsible for creating the root resource. It isn't done
         # at __init__ time because we need to reference the parent's basedir.
-        htmldir = os.path.join(self.parent.basedir, "public_html")
+        htmldir = os.path.abspath(os.path.join(self.master.basedir, self.public_html))
         if os.path.isdir(htmldir):
             log.msg("WebStatus using (%s)" % htmldir)
         else:
@@ -472,6 +566,10 @@ class WebStatus(service.MultiService):
         for name, child_resource in self.childrenToBeAdded.iteritems():
             root.putChild(name, child_resource)
 
+        status = self.getStatus()
+        root.putChild("rss", Rss20StatusResource(status))
+        root.putChild("atom", Atom10StatusResource(status))
+
         self.site.resource = root
 
     def putChild(self, name, child_resource):
@@ -492,17 +590,39 @@ class WebStatus(service.MultiService):
         return service.MultiService.stopService(self)
 
     def getStatus(self):
-        return self.parent.getStatus()
+        return self.master.getStatus()
+
     def getControl(self):
         if self.allowForce:
-            return IControl(self.parent)
+            return IControl(self.master)
         return None
 
+    def getChangeSvc(self):
+        return self.master.change_svc
+
     def getPortnum(self):
         # this is for the benefit of unit tests
         s = list(self)[0]
         return s._port.getHost().port
 
+    def isUsingUserPasswd(self):
+        """Returns boolean to indicate if this WebStatus uses authentication"""
+        if self.auth:
+            return True
+        return False
+
+    def authUser(self, user, passwd):
+        """Check that user/passwd is a valid user/pass tuple and can should be
+        allowed to perform the action. If this WebStatus is not password
+        protected, this function returns False."""
+        if not self.isUsingUserPasswd():
+            return False
+        if self.auth.authenticate(user, passwd):
+            return True
+        log.msg("Authentication failed for '%s': %s" % (user,
+                                                        self.auth.errmsg()))
+        return False
+
 # resources can get access to the IStatus by calling
 # request.site.buildbot_service.getStatus()
 
@@ -511,8 +631,8 @@ class WebStatus(service.MultiService):
 # always redirects to a WaterfallStatusResource, and the old arguments are
 # mapped into the new resource-tree approach. In the normal WebStatus, the
 # root resource either redirects the browser to /waterfall or serves
-# BASEDIR/public_html/index.html, and favicon/robots.txt are provided by
-# having the admin write actual files into BASEDIR/public_html/ .
+# PUBLIC_HTML/index.html, and favicon/robots.txt are provided by
+# having the admin write actual files into PUBLIC_HTML/ .
 
 # note: we don't use a util.Redirect here because HTTP requires that the
 # Location: header provide an absolute URI, and it's non-trivial to figure
@@ -555,7 +675,7 @@ class Waterfall(WebStatus):
                 pass
             else:
                 data = open(css, "rb").read()
-                self.putChild("buildbot.css", static.Data(data, "text/plain"))
+                self.putChild("buildbot.css", static.Data(data, "text/css"))
         self.favicon = favicon
         self.robots_txt = robots_txt
         if favicon:
