@@ -6,7 +6,7 @@ from twisted.internet import defer, reactor
 import urllib, time
 from twisted.python import log
 from buildbot.status.web.base import HtmlResource, make_row, make_stop_form, \
-     css_classes, path_to_builder, path_to_slave
+     css_classes, path_to_builder, path_to_slave, make_name_user_passwd_form
 
 from buildbot.status.web.tests import TestsResource
 from buildbot.status.web.step import StepsResource
@@ -49,9 +49,10 @@ class StatusResourceBuild(HtmlResource):
 
             if self.build_control is not None:
                 stopURL = urllib.quote(req.childLink("stop"))
-                data += make_stop_form(stopURL)
+                data += make_stop_form(stopURL, self.isUsingUserPasswd(req))
 
         if b.isFinished():
+            # Results map loosely to css_classes
             results = b.getResults()
             data += "<h2>Results:</h2>\n"
             text = " ".join(b.getText())
@@ -103,26 +104,37 @@ class StatusResourceBuild(HtmlResource):
 #        for name, target in urls.items():
 #            text.append('[<a href="%s" class="%s">%s</a>]' %
 #                        (target, ex_url_class, html.escape(name)))
-        if b.getLogs():
-            data += "<ol>\n"
-            for s in b.getSteps():
-                name = s.getName()
-                data += (" <li><a href=\"%s\">%s</a> [%s]\n"
-                         % (req.childLink("steps/%s" % urllib.quote(name)),
-                            name,
-                            " ".join(s.getText())))
-                if s.getLogs():
-                    data += "  <ol>\n"
-                    for logfile in s.getLogs():
-                        logname = logfile.getName()
-                        logurl = req.childLink("steps/%s/logs/%s" %
-                                               (urllib.quote(name),
-                                                urllib.quote(logname)))
-                        data += ("   <li><a href=\"%s\">%s</a></li>\n" %
-                                 (logurl, logfile.getName()))
-                    data += "  </ol>\n"
-                data += " </li>\n"
-            data += "</ol>\n"
+        data += "<ol>\n"
+        for s in b.getSteps():
+            name = s.getName()
+            time_to_run = 0
+            (start, end) = s.getTimes()
+            if start and end:
+              time_to_run = end - start
+            if s.isFinished():
+                css_class = css_classes[s.getResults()[0]]
+            elif s.isStarted():
+                css_class = "running"
+            else:
+                css_class = ""
+            data += (' <li><span class="%s"><a href=\"%s\">%s</a> [%s] [%d seconds]</span>\n'
+                     % (css_class, 
+                        req.childLink("steps/%s" % urllib.quote(name)),
+                        name,
+                        " ".join(s.getText()),
+                        time_to_run))
+            if s.getLogs():
+                data += "  <ol>\n"
+                for logfile in s.getLogs():
+                    logname = logfile.getName()
+                    logurl = req.childLink("steps/%s/logs/%s" %
+                                           (urllib.quote(name),
+                                            urllib.quote(logname)))
+                    data += ("   <li><a href=\"%s\">%s</a></li>\n" %
+                             (logurl, logfile.getName()))
+                data += "  </ol>\n"
+            data += " </li>\n"
+        data += "</ol>\n"
 
         data += "<h2>Build Properties:</h2>\n"
         data += "<table><tr><th valign=\"left\">Name</th><th valign=\"left\">Value</th><th valign=\"left\">Source</th></tr>\n"
@@ -154,6 +166,9 @@ class StatusResourceBuild(HtmlResource):
         if end:
             data += "<tr><td>End</td><td>%s</td></tr>\n" % time.ctime(end)
             data += "<tr><td>Elapsed</td><td>%s</td></tr>\n" % util.formatInterval(end - start)
+        else:
+            now = util.now()
+            data += "<tr><td>Elapsed</td><td>%s</td></tr>\n" % util.formatInterval(now - start)
         data += "</table>\n"
 
         if ss.changes:
@@ -181,10 +196,9 @@ class StatusResourceBuild(HtmlResource):
                          "after this build was started <b>will</b> be \n"
                          "included in a rebuild.</p>\n")
             rebuildURL = urllib.quote(req.childLink("rebuild"))
-            data += ('<form action="%s" class="command rebuild">\n'
+            data += ('<form method="post" action="%s" class="command rebuild">\n'
                      % rebuildURL)
-            data += make_row("Your name:",
-                             "<input type='text' name='username' />")
+            data += make_name_user_passwd_form(self.isUsingUserPasswd(req))
             data += make_row("Reason for re-running build:",
                              "<input type='text' name='comments' />")
             data += '<input type="submit" value="Rebuild" />\n'
@@ -216,15 +230,20 @@ class StatusResourceBuild(HtmlResource):
         return data
 
     def stop(self, req):
+        if self.isUsingUserPasswd(req):
+            if not self.authUser(req):
+                return Redirect("../../../authfailed")
         b = self.build_status
         c = self.build_control
         log.msg("web stopBuild of build %s:%s" % \
                 (b.getBuilder().getName(), b.getNumber()))
         name = req.args.get("username", ["<unknown>"])[0]
         comments = req.args.get("comments", ["<no reason specified>"])[0]
+        # html-quote both the username and comments, just to be safe
         reason = ("The web-page 'stop build' button was pressed by "
-                  "'%s': %s\n" % (name, comments))
-        c.stopBuild(reason)
+                  "'%s': %s\n" % (html.escape(name), html.escape(comments)))
+        if c:
+            c.stopBuild(reason)
         # we're at http://localhost:8080/svn-hello/builds/5/stop?[args] and
         # we want to go to: http://localhost:8080/svn-hello
         r = Redirect("../..")
@@ -233,6 +252,9 @@ class StatusResourceBuild(HtmlResource):
         return DeferredResource(d)
 
     def rebuild(self, req):
+        if self.isUsingUserPasswd(req):
+            if not self.authUser(req):
+                return Redirect("../../../authfailed")
         b = self.build_status
         bc = self.builder_control
         builder_name = b.getBuilder().getName()
@@ -240,7 +262,7 @@ class StatusResourceBuild(HtmlResource):
         name = req.args.get("username", ["<unknown>"])[0]
         comments = req.args.get("comments", ["<no reason specified>"])[0]
         reason = ("The web-page 'rebuild' button was pressed by "
-                  "'%s': %s\n" % (name, comments))
+                  "'%s': %s\n" % (html.escape(name), html.escape(comments)))
         if not bc or not b.isFinished():
             log.msg("could not rebuild: bc=%s, isFinished=%s"
                     % (bc, b.isFinished()))
