@@ -214,6 +214,7 @@ class CVS(Source):
 
     def __init__(self, cvsroot, cvsmodule,
                  global_options=[], branch=None, checkoutDelay=None,
+                 checkout_options=[],
                  login=None,
                  **kwargs):
 
@@ -266,7 +267,13 @@ class CVS(Source):
                                '-R'] will also assume the repository is
                                read-only (I assume this means it won't
                                use locks to insure atomic access to the
-                               ,v files)."""
+                               ,v files).
+                               
+        @type  checkout_options: list of strings
+        @param checkout_options: these arguments are inserted in the cvs
+                               command line, after 'checkout' but before
+                               branch or revision specifiers.
+                               """
 
         self.checkoutDelay = checkoutDelay
         self.branch = branch
@@ -275,6 +282,7 @@ class CVS(Source):
         self.addFactoryArguments(cvsroot=cvsroot,
                                  cvsmodule=cvsmodule,
                                  global_options=global_options,
+                                 checkout_options=checkout_options,
                                  branch=branch,
                                  checkoutDelay=checkoutDelay,
                                  login=login,
@@ -283,6 +291,7 @@ class CVS(Source):
         self.args.update({'cvsroot': cvsroot,
                           'cvsmodule': cvsmodule,
                           'global_options': global_options,
+                          'checkout_options':checkout_options,
                           'login': login,
                           })
 
@@ -351,7 +360,8 @@ class SVN(Source):
     name = 'svn'
 
     def __init__(self, svnurl=None, baseURL=None, defaultBranch=None,
-                 directory=None, username=None, password=None, **kwargs):
+                 directory=None, username=None, password=None,
+                 extra_args=None, **kwargs):
         """
         @type  svnurl: string
         @param svnurl: the URL which points to the Subversion server,
@@ -387,6 +397,7 @@ class SVN(Source):
         self.branch = defaultBranch
         self.username = username
         self.password = password
+        self.extra_args = extra_args
 
         Source.__init__(self, **kwargs)
         self.addFactoryArguments(svnurl=svnurl,
@@ -395,6 +406,7 @@ class SVN(Source):
                                  directory=directory,
                                  username=username,
                                  password=password,
+                                 extra_args=extra_args,
                                  )
 
         if not svnurl and not baseURL:
@@ -476,6 +488,9 @@ class SVN(Source):
             if self.username is not None: self.args['username'] = self.username
             if self.password is not None: self.args['password'] = self.password
 
+        if self.extra_args is not None:
+            self.args['extra_args'] = self.extra_args
+
         revstuff = []
         if branch is not None and branch != self.branch:
             revstuff.append("[branch]")
@@ -587,7 +602,10 @@ class Git(Source):
 
     name = "git"
 
-    def __init__(self, repourl, branch="master", **kwargs):
+    def __init__(self, repourl,
+                 branch="master",
+                 submodules=False,
+                 **kwargs):
         """
         @type  repourl: string
         @param repourl: the URL which points at the git repository
@@ -596,11 +614,21 @@ class Git(Source):
         @param branch: The branch or tag to check out by default. If
                        a build specifies a different branch, it will
                        be used instead of this.
+
+        @type  submodules: boolean
+        @param submodules: Whether or not to update (and initialize)
+                       git submodules.
+
         """
         Source.__init__(self, **kwargs)
-        self.addFactoryArguments(repourl=repourl, branch=branch)
+        self.addFactoryArguments(repourl=repourl,
+                                 branch=branch,
+                                 submodules=submodules,
+                                 )
         self.args.update({'repourl': repourl,
-                          'branch': branch})
+                          'branch': branch,
+                          'submodules' : submodules,
+                          })
 
     def computeSourceRevision(self, changes):
         if not changes:
@@ -795,6 +823,7 @@ class Bzr(Source):
     name = "bzr"
 
     def __init__(self, repourl=None, baseURL=None, defaultBranch=None,
+                 forceSharedRepo=None,
                  **kwargs):
         """
         @type  repourl: string
@@ -814,6 +843,15 @@ class Bzr(Source):
                               explicitly. It will simply be appended to
                               C{baseURL} and the result handed to the
                               'bzr checkout pull' command.
+
+
+        @param forceSharedRepo: Boolean, defaults to False. If set to True,
+                                the working directory will be made into a
+                                bzr shared repository if it is not already.
+                                Shared repository greatly reduces the amount
+                                of history data that needs to be downloaded
+                                if not using update/copy mode, or if using
+                                update/copy mode with multiple branches.
         """
         self.repourl = repourl
         self.baseURL = baseURL
@@ -822,7 +860,9 @@ class Bzr(Source):
         self.addFactoryArguments(repourl=repourl,
                                  baseURL=baseURL,
                                  defaultBranch=defaultBranch,
+                                 forceSharedRepo=forceSharedRepo
                                  )
+        self.args.update({'forceSharedRepo': forceSharedRepo})
         if (not repourl and not baseURL) or (repourl and baseURL):
             raise ValueError("you must provide exactly one of repourl and"
                              " baseURL")
@@ -849,7 +889,9 @@ class Bzr(Source):
 
         revstuff = []
         if branch is not None and branch != self.branch:
-            revstuff.append("[branch]")
+            revstuff.append("[" + branch + "]")
+        if revision is not None:
+            revstuff.append("r%s" % revision)
         self.description.extend(revstuff)
         self.descriptionDone.extend(revstuff)
 
@@ -863,7 +905,7 @@ class Mercurial(Source):
     name = "hg"
 
     def __init__(self, repourl=None, baseURL=None, defaultBranch=None,
-                 branchType='dirname', **kwargs):
+                 branchType='dirname', clobberOnBranchChange=True, **kwargs):
         """
         @type  repourl: string
         @param repourl: the URL which points at the Mercurial repository.
@@ -891,16 +933,23 @@ class Mercurial(Source):
                            the branch name should be appended to the C{baseURL}
                            or the branch is a mercurial named branch and can be
                            found within the C{repourl}
+
+        @param clobberOnBranchChange: boolean, defaults to True. If set and
+                                      using inrepos branches, clobber the tree
+                                      at each branch change. Otherwise, just
+                                      update to the branch.
         """
         self.repourl = repourl
         self.baseURL = baseURL
         self.branch = defaultBranch
         self.branchType = branchType
+        self.clobberOnBranchChange = clobberOnBranchChange
         Source.__init__(self, **kwargs)
         self.addFactoryArguments(repourl=repourl,
                                  baseURL=baseURL,
                                  defaultBranch=defaultBranch,
                                  branchType=branchType,
+                                 clobberOnBranchChange=clobberOnBranchChange,
                                  )
         if (not repourl and not baseURL) or (repourl and baseURL):
             raise ValueError("you must provide exactly one of repourl and"
@@ -919,9 +968,11 @@ class Mercurial(Source):
             if branch:
                 self.args['branch'] = branch
         else:
-            self.args['repourl'] = self.baseURL + branch
+            self.args['repourl'] = self.baseURL + (branch or '')
         self.args['revision'] = revision
         self.args['patch'] = patch
+        self.args['clobberOnBranchChange'] = self.clobberOnBranchChange
+        self.args['branchType'] = self.branchType
 
         revstuff = []
         if branch is not None and branch != self.branch:
