@@ -13,6 +13,7 @@ from buildbot.process.base import BuildRequest, Build
 from buildbot.process.buildstep import BuildStep
 from buildbot.sourcestamp import SourceStamp
 from buildbot.status import builder
+from buildbot.process.properties import Properties
 
 
 
@@ -144,7 +145,7 @@ class RunMixin:
     def requestBuild(self, builder):
         # returns a Deferred that fires with an IBuildStatus object when the
         # build is finished
-        req = BuildRequest("forced build", SourceStamp())
+        req = BuildRequest("forced build", SourceStamp(), 'test_builder')
         self.control.getBuilder(builder).requestBuild(req)
         return req.waitUntilFinished()
 
@@ -152,7 +153,7 @@ class RunMixin:
         if bs.getResults() != builder.SUCCESS:
             log.msg("failUnlessBuildSucceeded noticed that the build failed")
             self.logBuildResults(bs)
-        self.failUnless(bs.getResults() == builder.SUCCESS)
+        self.failUnlessEqual(bs.getResults(), builder.SUCCESS)
         return bs # useful for chaining
 
     def logBuildResults(self, bs):
@@ -194,8 +195,8 @@ class RunMixin:
         log.msg("doing shutdownAllSlaves")
         dl = []
         for slave in self.slaves.values():
+            slave.stopService()
             dl.append(slave.waitUntilDisconnected())
-            dl.append(defer.maybeDeferred(slave.stopService))
         d = defer.DeferredList(dl)
         d.addCallback(self._shutdownAllSlavesDone)
         return d
@@ -275,6 +276,12 @@ def fake_slaveVersion(command, oldversion=None):
     from buildbot.slave.registry import commandRegistry
     return commandRegistry[command]
 
+class FakeBuildMaster:
+    properties = Properties(masterprop="master")
+
+class FakeBotMaster:
+    parent = FakeBuildMaster()
+
 def makeBuildStep(basedir, step_class=BuildStep, **kwargs):
     bss = setupBuildStepStatus(basedir)
 
@@ -282,13 +289,15 @@ def makeBuildStep(basedir, step_class=BuildStep, **kwargs):
     setup = {'name': "builder1", "slavename": "bot1",
              'builddir': "builddir", 'factory': None}
     b0 = Builder(setup, bss.getBuild().getBuilder())
-    br = BuildRequest("reason", ss)
+    b0.botmaster = FakeBotMaster()
+    br = BuildRequest("reason", ss, 'test_builder')
     b = Build([br])
     b.setBuilder(b0)
     s = step_class(**kwargs)
     s.setBuild(b)
     s.setStepStatus(bss)
-    b.setupStatus(bss.getBuild())
+    b.build_status = bss.getBuild()
+    b.setupProperties()
     s.slaveVersion = fake_slaveVersion
     return s
 
@@ -300,7 +309,7 @@ def findDir():
 class SignalMixin:
     sigchldHandler = None
     
-    def setUpClass(self):
+    def setUpSignalHandler(self):
         # make sure SIGCHLD handler is installed, as it should be on
         # reactor.run(). problem is reactor may not have been run when this
         # test runs.
@@ -308,7 +317,7 @@ class SignalMixin:
             self.sigchldHandler = signal.signal(signal.SIGCHLD,
                                                 reactor._handleSigchld)
 
-    def tearDownClass(self):
+    def tearDownSignalHandler(self):
         if self.sigchldHandler:
             signal.signal(signal.SIGCHLD, self.sigchldHandler)
 
@@ -330,6 +339,12 @@ class FakeSlaveBuilder:
 class SlaveCommandTestBase(SignalMixin):
     usePTY = False
 
+    def setUp(self):
+        self.setUpSignalHandler()
+
+    def tearDown(self):
+        self.tearDownSignalHandler()
+
     def setUpBuilder(self, basedir):
         if not os.path.exists(basedir):
             os.mkdir(basedir)
@@ -476,11 +491,14 @@ class SetTestFlagStep(BuildStep):
     """
     def __init__(self, flagname='flag', value=1, **kwargs):
         BuildStep.__init__(self, **kwargs)
+        self.addFactoryArguments(flagname=flagname, value=value)
+
         self.flagname = flagname
         self.value = value
 
     def start(self):
-        _flags[self.flagname] = self.value
+        properties = self.build.getProperties()
+        _flags[self.flagname] = properties.render(self.value)
         self.finished(builder.SUCCESS)
 
 class TestFlagMixin:
