@@ -1,9 +1,15 @@
 # -*- test-case-name: buildbot.test.test_mailparse -*-
 
+import time
+import os
 from twisted.trial import unittest
 from twisted.python import util
 from buildbot.changes import mail
 
+import sys
+if sys.version_info[:3] < (2,4,0):
+    from sets import Set as set
+
 class TestFreshCVS(unittest.TestCase):
 
     def get(self, msg):
@@ -291,3 +297,117 @@ class TestSVNCommitEmail(unittest.TestCase):
                              "Bug 4696: consolidated fixes for timeout bugs\n")
 
 
+class TestBzrLaunchpadEmail(unittest.TestCase):
+    def get(self, msg, **kwargs):
+        msg = util.sibpath(__file__, "mail/" + msg)
+        s = mail.BzrLaunchpadEmailMaildirSource(None, None, **kwargs)
+        if not os.path.exists(msg):
+            raise unittest.SkipTest("'%s' not found" % msg)
+        return s.parse_file(open(msg, "r"))
+
+    # Modified file
+    def test1(self):
+        c = self.get("launchpad.1")
+        self.failUnlessEqual(c.revision, "2700")
+        self.failUnlessEqual(c.who, "Kristian Nielsen <knielsen@loke>")
+        self.failUnlessEqual(c.branch, "lp:~knielsen/maria/tmp-buildbot-test")
+        self.failUnlessEqual(time.gmtime(c.when)[0:6], (2009, 5, 15, 6, 40, 43))
+        self.failUnlessEqual(c.comments, "test commit 2\n")
+        self.failUnlessEqual(set(c.files), set(["README MODIFIED",
+                                                "Makefile.am MODIFIED"]))
+
+    # Added file and branchMap match with lp:
+    def test2(self):
+        c = self.get("launchpad.2",
+                     branchMap = { "lp:~knielsen/maria/tmp-buildbot-test" : "5.1",
+                                   "foobar" : "baz" },
+                     defaultBranch = "bogux")
+        self.failUnlessEqual(c.revision, "2701")
+        self.failUnlessEqual(c.who, "Kristian Nielsen <knielsen@loke>")
+        self.failUnlessEqual(c.branch, "5.1")
+        self.failUnlessEqual(time.gmtime(c.when)[0:6], (2009, 5, 15, 8, 35, 43))
+        self.failUnlessEqual(c.comments, "test add file\n")
+        self.failUnlessEqual(set(c.files), set(["test-add-file ADDED"]))
+
+    # Rename file and branchMap match without lp:
+    def test3(self):
+        c = self.get("launchpad.3",
+                     branchMap = { "~knielsen/maria/tmp-buildbot-test" : "5.1",
+                                   "foobar" : "baz" },
+                     defaultBranch = "bogux")
+        self.failUnlessEqual(c.revision, "2702")
+        self.failUnlessEqual(c.who, "Kristian Nielsen <knielsen@loke>")
+        self.failUnlessEqual(c.branch, "5.1")
+        self.failUnlessEqual(time.gmtime(c.when)[0:6], (2009, 5, 15, 8, 36, 24))
+        self.failUnlessEqual(c.comments, "test rename file\n")
+        self.failUnlessEqual(set(c.files), set(["test-add-file RENAMED test-rename-file"]))
+
+    # Remove file and branchMap fallback to defaultBranch
+    def test4(self):
+        c = self.get("launchpad.4",
+                     branchMap = { "lp::~knielsen/maria/tmp-buildbot-test" : "5.1",
+                                   "foobar" : "baz" },
+                     defaultBranch = "db")
+        self.failUnlessEqual(c.revision, "2703")
+        self.failUnlessEqual(c.who, "Kristian Nielsen <knielsen@loke>")
+        self.failUnlessEqual(c.branch, "db")
+        self.failUnlessEqual(time.gmtime(c.when)[0:6], (2009, 5, 15, 8, 36, 35))
+        self.failUnlessEqual(c.comments, "test remove file\n")
+        self.failUnlessEqual(set(c.files), set(["test-rename-file REMOVED"]))
+
+    # MIME decoding of message body and branch name fallback to repository name
+    def test5(self):
+        c = self.get("launchpad.5",
+                     branchMap = { "lp::~knielsen/maria/tmp-buildbot-test" : "5.1",
+                                   "foobar" : "baz" })
+        self.failUnlessEqual(c.revision, "2704")
+        self.failUnlessEqual(c.who, "Kristian Nielsen <knielsen@loke>")
+        self.failUnlessEqual(c.branch, "lp:~knielsen/maria/tmp-buildbot-test")
+        self.failUnlessEqual(time.gmtime(c.when)[0:6], (2009, 5, 15, 8, 39, 58))
+        self.failUnlessEqual(c.comments, "10\n")
+        self.failUnlessEqual(set(c.files), set(["tricky=>rename ADDED"]))
+
+    # Both rename and modify in same message.
+    def test6(self):
+        c = self.get("launchpad.6")
+        self.failUnlessEqual(c.revision, "2706")
+        self.failUnlessEqual(c.who, "Kristian Nielsen <knielsen@loke>")
+        self.failUnlessEqual(c.branch, "lp:~knielsen/maria/tmp-buildbot-test")
+        self.failUnlessEqual(time.gmtime(c.when)[0:6], (2009, 5, 15, 8, 47, 13))
+        self.failUnlessEqual(c.comments, "rename+mod\n")
+        self.failUnlessEqual(set(c.files),
+                             set(["renamed with => tricky <= name RENAMED better_name.txt",
+                                  "better_name.txt MODIFIED"]))
+
+    # Test equality comparison (for working config reload).
+    def test_bzr_mail_reload(self):
+        s0 = mail.BzrLaunchpadEmailMaildirSource("/dir1")
+        s1 = mail.BzrLaunchpadEmailMaildirSource("/dir2")
+        self.failIfEqual(s0,s1)
+        s2 = mail.BzrLaunchpadEmailMaildirSource("/dir1", prefix = "lp:")
+        self.failIfEqual(s0,s2)
+        s3 = mail.BzrLaunchpadEmailMaildirSource("/dir1", prefix = "pl:")
+        self.failIfEqual(s2,s3)
+        s4 = mail.BzrLaunchpadEmailMaildirSource("/dir1",
+                                                 branchMap = { "a" : "A" })
+        self.failIfEqual(s0,s4)
+        s5 = mail.BzrLaunchpadEmailMaildirSource("/dir1",
+                                                 branchMap = { "a" : "A", "b" : "B" })
+        self.failIfEqual(s4,s5)
+        s6 = mail.BzrLaunchpadEmailMaildirSource("/dir1",
+                                                 defaultBranch = "b1")
+        self.failIfEqual(s0,s6)
+        s7 = mail.BzrLaunchpadEmailMaildirSource("/dir1",
+                                                 defaultBranch = "b2")
+        self.failIfEqual(s6,s7)
+        s8 = mail.BzrLaunchpadEmailMaildirSource("/dir1",
+                                                 prefix = "lp",
+                                                 branchMap = { "c" : "C" },
+                                                 defaultBranch = "b3")
+        self.failIfEqual(s6,s8)
+        s9 = mail.BzrLaunchpadEmailMaildirSource("/dir1",
+                                                 prefix = "lp",
+                                                 branchMap = { "c" : "C" },
+                                                 defaultBranch = "b3")
+        s9.dummy = 42
+        self.assertEqual(s8,s9)
